# generated by datamodel-codegen:
#   filename:  swagger.json
#   timestamp: 2023-03-19T19:24:29+00:00

from __future__ import annotations

from typing import Optional

import cherwell_pydantic_api.types as ct
from cherwell_pydantic_api.generated_api_utils import ApiBaseModel

from .. import CherwellObjectID


class CherwellNameValuePair(ApiBaseModel):
    name: Optional[str] = None
    valueObject: Optional[CherwellObjectID] = None
    valueString: Optional[str] = None
    category: Optional[str] = None
    description: Optional[str] = None
    displayShowsValue: Optional[bool] = None
    specialUseFlag: Optional[bool] = None
    displayString: Optional[str] = None
