{% for decorator in decorators -%}
{{ decorator }}
{% endfor -%}
{% if class_name  == 'HttpStatusCode' %}
from cherwell_pydantic_api.generated_api_utils import {{ class_name }}
{% else %}
class {{ class_name }}({{ base_class }}):
{%- if description %}
    """
    {{ description | indent(4) }}
    """
{%- endif %}
{%- for field in fields %}
    {{ field.name }} = {{ field.default }}
    {%- if field.docstring %}
    """
    {{ field.docstring | indent(4) }}
    """
    {%- endif %}
{%- endfor -%}
{% endif %}
