


class BusinessObjectBase(object):
    """docstring for BusinessObjectBase"""
    def __init__(self, connection):
        super(BusinessObjectBase, self).__init__()
        self._conn = connection
        
